import { EventsHandler, IEventHandler } from '@nestjs/cqrs';
import { TransactionDataService } from 'src/modules/transaction/transaction/data/services/transaction-data.service';
import {
  TransactionChangeStatusEvent,
  TransactionCreateQueueEvent,
} from 'src/modules/transaction/transaction/domain/entities/event/transaction-change-status.event';
import { TicketDataService } from '../../data/services/ticket.service';
import { QueueOrder } from '../../domain/entities/order.entity';
import { QueueTicket } from '../../domain/entities/ticket.entity';
import { QueueItem } from '../../domain/entities/queue-item.entity';
import * as moment from 'moment';
import { QueueOrchestrator } from '../../domain/queue.orchestrator';
import { TransactionUserType } from 'src/modules/transaction/transaction/constants';

@EventsHandler(TransactionChangeStatusEvent, TransactionCreateQueueEvent)
export class QueueTransactionHandler
  implements IEventHandler<TransactionChangeStatusEvent>
{
  constructor(
    private dataService: TransactionDataService,
    private queueService: TicketDataService,
    private orchestrator: QueueOrchestrator,
  ) {}

  async handle(event: TransactionChangeStatusEvent) {
    const process_data = event.data.data;

    /**
     * If data still in process (not settled) then don't create the queue order
     */
    if (process_data?.status != 'settled') return;

    const transaction = await this.dataService.getOneByOptions({
      where: {
        id: event.data.id,
      },
      relations: ['customer_category', 'items', 'items.item'],
    });

    const date = transaction.booking_date ?? transaction.invoice_date;
    const queue_date = moment(date, 'YYYY-MM-DD').unix();

    const { id, customer_name, customer_phone, invoice_code } = transaction;

    const customerOrder = {
      code: invoice_code,
      customer: customer_name,
      phone: customer_phone,
      date: queue_date * 1000,
      transaction_id: id,
    };

    const items = transaction.items
      .filter((item) => item.item.use_queue)
      .map<QueueItem>((item) => {
        return {
          item_id: item.item_id,
          qty: item.qty,
        };
      });

    const ticket: QueueTicket = { ...customerOrder, items };
    const order: QueueOrder = { ...customerOrder, tickets: [ticket] };

    const queueOrder = await this.queueService.createQueueOrder(order);
    if (
      transaction.customer_category?.has_vip_pass ||
      transaction.customer_type === TransactionUserType.VIP
    ) {
      queueOrder.tickets.forEach((ticket) => {
        const ticket_id = ticket.id;
        ticket.items.forEach((item) => {
          const item_id = item.item_id;

          for (let i = 0; i < item.qty; i++) {
            const payload = {
              item_id,
              ticket_id,
              qty: 1,
            };

            this.orchestrator.create(payload);
          }
        });
      });
    }
  }
}
